from edc_navbar import Navbar, NavbarItem, site_navbars

navbar = Navbar(name="edc_consent")

navbar.append_item(
    NavbarItem(
        name="consent",
        label="Consent",
        fa_icon="fa-solid fa-user-circle",
        url_name="edc_consent:home_url",
        codename="edc_consent.nav_consent",
    )
)

site_navbars.register(navbar)



CLINIC_NUMBER = "clinic_no"
CONSENT_WITHDRAWAL = "consent_withdrawn"
DEFAULT_CONSENT_GROUP = "default"
HOSPITAL_NUMBER = "hospital_no"
MOBILE_NUMBER = "mobile_no"
NATIONAL_IDENTIFIER = "national_identifier"



from django.core.validators import RegexValidator
from django.db import models
from django.utils.html import format_html
from django_crypto_fields.fields import (
    EncryptedCharField,
    FirstnameField,
    LastnameField,
)
from django_crypto_fields.models import CryptoMixin
from edc_constants.choices import GENDER_UNDETERMINED
from edc_model.models import NameFieldsModelMixin
from edc_model_fields.fields import IsDateEstimatedField

from ..validators import FullNameValidator


class BaseFieldsMixin(models.Model):

    initials = EncryptedCharField(
        validators=[
            RegexValidator(
                regex=r"^[A-Z]{2,3}$",
                message="Ensure initials consist of letters only in upper case, no spaces.",
            )
        ],
        null=True,
        blank=False,
    )

    dob = models.DateField(verbose_name="Date of birth", null=True, blank=False)

    is_dob_estimated = IsDateEstimatedField(
        verbose_name="Is date of birth estimated?", null=True, blank=False
    )

    gender = models.CharField(
        verbose_name="Gender",
        choices=GENDER_UNDETERMINED,
        max_length=1,
        null=True,
        blank=False,
    )

    guardian_name = LastnameField(
        verbose_name="Guardian's last and first name",
        validators=[FullNameValidator()],
        blank=True,
        null=True,
        help_text=format_html(
            "Required only if participant is a minor.<BR>"
            "Format is 'LASTNAME, FIRSTNAME'. "
            "All uppercase separated by a comma."
        ),
    )

    subject_type = models.CharField(max_length=25)

    class Meta:
        abstract = True


class FullNamePersonalFieldsMixin(
    CryptoMixin, NameFieldsModelMixin, BaseFieldsMixin, models.Model
):
    class Meta:
        abstract = True


class PersonalFieldsMixin(CryptoMixin, BaseFieldsMixin, models.Model):

    first_name = FirstnameField(
        null=True,
        blank=False,
        validators=[
            RegexValidator(
                regex=r"^([A-Z]+$|[A-Z]+\ [A-Z]+)$",
                message="Ensure name consist of letters only in upper case",
            )
        ],
        help_text="Use UPPERCASE letters only.",
    )

    last_name = LastnameField(
        verbose_name="Surname",
        null=True,
        blank=False,
        validators=[
            RegexValidator(
                regex=r"^([A-Z]+$|[A-Z]+\ [A-Z]+)$",
                message="Ensure name consist of letters only in upper case",
            )
        ],
        help_text="Use UPPERCASE letters only.",
    )

    class Meta:
        abstract = True



from .utils import get_consent_model_name, get_reconsent_model_name

consent_codenames = []
models = [get_consent_model_name()]
if get_reconsent_model_name():
    models.append(get_reconsent_model_name())
for model in models:
    for action in ["view_", "add_", "change_", "delete_", "view_historical"]:
        consent_codenames.append(f".{action}".join(model.split(".")))

navbar_codenames = [
    "edc_consent.nav_consent",
]

navbar_tuples = []
for codename in navbar_codenames:
    navbar_tuples.append((codename, f"Can access {codename.split('.')[1]}"))

consent_codenames.extend(navbar_codenames)
consent_codenames.sort()



from django.db import models


class VerificationFieldsMixin(models.Model):

    """A fields mixin for models that are verified against a
    paper document, such as an ICF.

    These fields are updated through an Admin action
    """

    is_verified = models.BooleanField(default=False, editable=False)

    is_verified_datetime = models.DateTimeField(null=True, editable=False)

    verified_by = models.CharField(max_length=25, null=True, editable=False)

    class Meta:
        abstract = True



from datetime import timedelta
from uuid import uuid4

from dateutil.relativedelta import relativedelta
from django import forms
from django.contrib.sites.models import Site
from django.forms import model_to_dict
from django.test import TestCase, override_settings
from edc_constants.constants import FEMALE, MALE, NO, NOT_APPLICABLE, YES
from edc_crf.crf_form_validator_mixins import BaseFormValidatorMixin
from edc_form_validators import FormValidator, FormValidatorMixin
from edc_protocol import Protocol
from edc_utils import age, get_utcnow
from faker import Faker
from model_bakery import baker

from edc_consent.consent import Consent
from edc_consent.modelform_mixins import ConsentModelFormMixin
from edc_consent.site_consents import site_consents

from ...form_validators import SubjectConsentFormValidatorMixin
from ..models import SubjectConsent, SubjectScreening

fake = Faker()


class SubjectConsentFormValidator(
    SubjectConsentFormValidatorMixin, BaseFormValidatorMixin, FormValidator
):
    pass


class SubjectConsentForm(ConsentModelFormMixin, FormValidatorMixin, forms.ModelForm):

    form_validator_cls = SubjectConsentFormValidator

    screening_identifier = forms.CharField(
        label="Screening identifier",
        widget=forms.TextInput(attrs={"readonly": "readonly"}),
    )

    class Meta:
        model = SubjectConsent
        fields = "__all__"


@override_settings(
    EDC_PROTOCOL_STUDY_OPEN_DATETIME=get_utcnow() - relativedelta(years=5),
    EDC_PROTOCOL_STUDY_CLOSE_DATETIME=get_utcnow() + relativedelta(years=1),
    EDC_AUTH_SKIP_SITE_AUTHS=True,
    EDC_AUTH_SKIP_AUTH_UPDATER=False,
)
class TestConsentForm(TestCase):
    def setUp(self):
        site_consents.registry = {}
        self.study_open_datetime = Protocol().study_open_datetime
        self.study_close_datetime = Protocol().study_close_datetime

        self.consent_factory(
            start=self.study_open_datetime,
            end=self.study_open_datetime + timedelta(days=50),
            version="1.0",
        )
        self.consent_factory(
            start=self.study_open_datetime + timedelta(days=51),
            end=self.study_open_datetime + timedelta(days=100),
            version="2.0",
        )
        self.consent_factory(
            start=self.study_open_datetime + timedelta(days=101),
            end=self.study_open_datetime + timedelta(days=150),
            version="3.0",
            updates_versions="1.0, 2.0",
        )
        self.dob = self.study_open_datetime - relativedelta(years=25)

    @staticmethod
    def consent_factory(**kwargs):
        options = dict(
            start=kwargs.get("start"),
            end=kwargs.get("end"),
            gender=kwargs.get("gender", ["M", "F"]),
            updates_versions=kwargs.get("updates_versions", []),
            version=kwargs.get("version", "1"),
            age_min=kwargs.get("age_min", 16),
            age_max=kwargs.get("age_max", 64),
            age_is_adult=kwargs.get("age_is_adult", 18),
        )
        model = kwargs.get("model", "edc_consent.subjectconsent")
        consent = Consent(model, **options)
        site_consents.register(consent)
        return consent

    def cleaned_data(self, **kwargs):
        cleaned_data = dict(
            consent_datetime=self.study_open_datetime,
            dob=self.study_open_datetime - relativedelta(years=25),
            first_name="THING",
            last_name="ONE",
            initials="TO",
            gender=MALE,
            identity="12315678",
            confirm_identity="12315678",
            identity_type="passport",
            is_dob_estimated="-",
            language="en",
            is_literate=YES,
            is_incarcerated=NO,
            study_questions=YES,
            consent_reviewed=YES,
            consent_copy=YES,
            assessment_score=YES,
            consent_signature=YES,
            site=Site.objects.get_current(),
            legal_marriage=NO,
            marriage_certificate=NOT_APPLICABLE,
            subject_type="subject",
            citizen=YES,
            subject_identifier=uuid4().hex,
        )
        cleaned_data.update(**kwargs)
        return cleaned_data

    def prepare_subject_consent(
        self,
        dob=None,
        consent_datetime=None,
        first_name=None,
        last_name=None,
        initials=None,
        gender=None,
        screening_identifier=None,
        identity=None,
        confirm_identity=None,
        age_in_years=None,
        is_literate=None,
        witness_name=None,
        create_subject_screening=None,
    ):
        create_subject_screening = (
            True if create_subject_screening is None else create_subject_screening
        )
        consent_datetime = consent_datetime or self.study_open_datetime
        dob = dob or self.study_open_datetime - relativedelta(years=25)
        gender = gender or FEMALE
        screening_identifier = screening_identifier or "ABCD"
        age_in_years = age_in_years or age(dob, reference_dt=consent_datetime).years
        initials = initials or "XX"
        if create_subject_screening:
            SubjectScreening.objects.create(
                age_in_years=age_in_years,
                initials=initials,
                gender=gender,
                screening_identifier=screening_identifier,
                report_datetime=consent_datetime,
                eligibility_datetime=consent_datetime,
            )
        subject_consent = baker.prepare_recipe(
            "edc_consent.subjectconsent",
            dob=dob,
            consent_datetime=consent_datetime,
            first_name=first_name or "XXXXXX",
            last_name=last_name or "XXXXXX",
            initials=initials,
            gender=gender,
            identity=identity or "123456789",
            confirm_identity=confirm_identity or "123456789",
            screening_identifier=screening_identifier,
            is_literate=is_literate or YES,
            witness_name=witness_name,
        )
        return subject_consent

    def test_base_form_is_valid(self):
        """Asserts baker defaults validate."""
        options = dict(
            dob=self.dob,
            consent_datetime=self.study_open_datetime,
            first_name="ERIK",
            last_name="THEPLEEB",
            initials="ET",
            screening_identifier="ABCD1",
        )
        subject_consent = self.prepare_subject_consent(**options)
        opts = SubjectConsentForm._meta
        data = model_to_dict(subject_consent, opts.fields, opts.exclude)
        form = SubjectConsentForm(
            data=data,
            initial=dict(screening_identifier=data.get("screening_identifier")),
            instance=SubjectConsent(),
        )
        self.assertTrue(form.is_valid())

    def test_base_form_catches_consent_datetime_before_study_open(self):
        options = dict(
            consent_datetime=self.study_open_datetime + relativedelta(days=1),
            dob=self.dob,
            first_name="ERIK",
            last_name="THEPLEEB",
            initials="ET",
            screening_identifier="ABCD1",
        )
        subject_consent = self.prepare_subject_consent(**options)
        opts = SubjectConsentForm._meta
        data = model_to_dict(subject_consent, opts.fields, opts.exclude)
        form = SubjectConsentForm(
            data=data,
            initial=dict(screening_identifier=data.get("screening_identifier")),
            instance=SubjectConsent(),
        )
        form.is_valid()
        self.assertEqual(form._errors, {})

        # change consent_datetime to before the consent period
        options.update(consent_datetime=self.study_open_datetime - relativedelta(days=1))
        subject_consent = self.prepare_subject_consent(
            **options, create_subject_screening=False
        )
        data = model_to_dict(subject_consent, opts.fields, opts.exclude)
        form = SubjectConsentForm(
            data=data,
            initial=dict(screening_identifier=data.get("screening_identifier")),
            instance=SubjectConsent(),
        )
        form.is_valid()
        self.assertIn("consent_datetime", form._errors)

    def test_base_form_identity_mismatch(self):
        options = dict(
            consent_datetime=self.study_open_datetime,
            dob=self.dob,
            first_name="ERIK",
            last_name="THEPLEEB",
            initials="ET",
            screening_identifier="ABCD1",
            identity="1",
            confirm_identity="2",
        )
        subject_consent = self.prepare_subject_consent(**options)
        opts = SubjectConsentForm._meta
        data = model_to_dict(subject_consent, opts.fields, opts.exclude)
        form = SubjectConsentForm(
            data=data,
            initial=dict(screening_identifier=data.get("screening_identifier")),
            instance=SubjectConsent(),
        )
        form.is_valid()
        self.assertIn("identity", form._errors)

    def test_base_form_identity_dupl(self):
        options = dict(
            consent_datetime=self.study_open_datetime,
            dob=self.dob,
            identity="123156788",
            confirm_identity="123156788",
            first_name="ERIK",
            last_name="THEPLEEB",
            initials="ET",
            screening_identifier="ABCD1",
        )
        subject_consent = self.prepare_subject_consent(**options)
        subject_consent.save()

        options = dict(
            consent_datetime=self.study_open_datetime,
            dob=self.dob,
            identity="123156788",
            confirm_identity="123156788",
            first_name="ERIK2",
            last_name="THEPLEEB2",
            initials="ET",
            screening_identifier="ABCD1XXX",
        )
        subject_consent = self.prepare_subject_consent(**options)
        opts = SubjectConsentForm._meta
        data = model_to_dict(subject_consent, opts.fields, opts.exclude)
        form = SubjectConsentForm(
            data=data,
            initial=dict(screening_identifier=data.get("screening_identifier")),
            instance=SubjectConsent(),
        )
        form.is_valid()
        self.assertIn("identity", form._errors)

    def test_base_form_guardian_and_dob1(self):
        """Asserts form for minor is not valid without guardian name."""
        options = dict(
            consent_datetime=self.study_open_datetime,
            dob=self.study_open_datetime - relativedelta(years=16),
            identity="123156788",
            confirm_identity="123156788",
            first_name="ERIK",
            last_name="THEPLEEB",
            initials="ET",
            screening_identifier="ABCD1",
        )
        subject_consent = self.prepare_subject_consent(**options)
        subject_consent.guardian_name = None
        opts = SubjectConsentForm._meta
        data = model_to_dict(subject_consent, opts.fields, opts.exclude)
        form = SubjectConsentForm(
            data=data,
            initial=dict(screening_identifier=data.get("screening_identifier")),
            instance=SubjectConsent(),
        )
        form.is_valid()
        self.assertIn("guardian_name", form._errors)

    def test_base_form_guardian_and_dob2(self):
        """Asserts form for minor is valid with guardian name."""
        options = dict(
            consent_datetime=self.study_open_datetime,
            dob=self.study_open_datetime - relativedelta(years=16),
            identity="123156788",
            confirm_identity="123156788",
            first_name="ERIK",
            last_name="THEPLEEB",
            initials="ET",
            screening_identifier="ABCD1",
        )
        subject_consent = self.prepare_subject_consent(**options)
        subject_consent.guardian_name = "SPOCK, YOUCOULDNTPRONOUNCEIT"
        opts = SubjectConsentForm._meta
        data = model_to_dict(subject_consent, opts.fields, opts.exclude)
        form = SubjectConsentForm(
            data=data,
            initial=dict(screening_identifier=data.get("screening_identifier")),
            instance=SubjectConsent(),
        )
        form.is_valid()
        self.assertEqual({}, form._errors)

    def test_base_form_guardian_and_dob4(self):
        """Asserts form for adult is not valid if guardian name
        specified.
        """
        options = dict(
            consent_datetime=self.study_open_datetime,
            dob=self.study_open_datetime - relativedelta(years=25),
            identity="123156788",
            confirm_identity="123156788",
            first_name="ERIK",
            last_name="THEPLEEB",
            initials="ET",
            screening_identifier="ABCD1",
        )
        subject_consent = self.prepare_subject_consent(**options)
        subject_consent.guardian_name = "SPOCK, YOUCOULDNTPRONOUNCEIT"
        opts = SubjectConsentForm._meta
        data = model_to_dict(subject_consent, opts.fields, opts.exclude)
        form = SubjectConsentForm(
            data=data,
            initial=dict(screening_identifier=data.get("screening_identifier")),
            instance=SubjectConsent(),
        )
        form.is_valid()
        self.assertIn("guardian_name", form._errors)

    def test_base_form_catches_dob_lower(self):
        options = dict(
            consent_datetime=self.study_open_datetime,
            dob=self.study_open_datetime - relativedelta(years=15),
            identity="123156788",
            confirm_identity="123156788",
            first_name="ERIK",
            last_name="THEPLEEB",
            initials="ET",
            screening_identifier="ABCD1",
        )
        subject_consent = self.prepare_subject_consent(**options)
        opts = SubjectConsentForm._meta
        data = model_to_dict(subject_consent, opts.fields, opts.exclude)
        form = SubjectConsentForm(
            data=data,
            initial=dict(screening_identifier=data.get("screening_identifier")),
            instance=SubjectConsent(),
        )
        form.is_valid()
        self.assertIn("dob", form._errors)

    def test_base_form_catches_dob_upper(self):
        options = dict(
            consent_datetime=self.study_open_datetime,
            dob=self.study_open_datetime - relativedelta(years=100),
            identity="123156788",
            confirm_identity="123156788",
            first_name="ERIK",
            last_name="THEPLEEB",
            initials="ET",
            screening_identifier="ABCD1",
        )
        subject_consent = self.prepare_subject_consent(**options)
        opts = SubjectConsentForm._meta
        data = model_to_dict(subject_consent, opts.fields, opts.exclude)
        form = SubjectConsentForm(
            data=data,
            initial=dict(screening_identifier=data.get("screening_identifier")),
            instance=SubjectConsent(),
        )
        form.is_valid()
        self.assertIn("dob", form._errors)

    def test_base_form_catches_gender_of_consent(self):
        site_consents.registry = {}
        self.consent_factory(
            start=self.study_open_datetime,
            end=self.study_open_datetime + timedelta(days=50),
            version="1.0",
            gender=[MALE],
            first_name="ERIK",
            last_name="THEPLEEB",
        )
        subject_consent = self.prepare_subject_consent(gender=MALE)
        opts = SubjectConsentForm._meta
        data = model_to_dict(subject_consent, opts.fields, opts.exclude)
        form = SubjectConsentForm(
            data=data,
            initial=dict(screening_identifier=data.get("screening_identifier")),
            instance=SubjectConsent(),
        )
        form.is_valid()
        self.assertEqual({}, form._errors)

        subject_consent = self.prepare_subject_consent(
            gender=FEMALE, create_subject_screening=False
        )
        opts = SubjectConsentForm._meta
        data = model_to_dict(subject_consent, opts.fields, opts.exclude)
        form = SubjectConsentForm(
            data=data,
            initial=dict(screening_identifier=data.get("screening_identifier")),
            instance=SubjectConsent(),
        )
        form.is_valid()
        self.assertIn("gender", form._errors)

    def test_base_form_catches_is_literate_and_witness(self):
        subject_consent = self.prepare_subject_consent(
            is_literate=NO,
            witness_name="",
        )
        opts = SubjectConsentForm._meta
        data = model_to_dict(subject_consent, opts.fields, opts.exclude)
        form = SubjectConsentForm(
            data=data,
            initial=dict(screening_identifier=data.get("screening_identifier")),
            instance=SubjectConsent(),
        )
        form.is_valid()
        self.assertIn("witness_name", form._errors)

        subject_consent = self.prepare_subject_consent(
            is_literate=NO,
            witness_name="BUBBA, BUBBA",
            create_subject_screening=False,
        )
        opts = SubjectConsentForm._meta
        data = model_to_dict(subject_consent, opts.fields, opts.exclude)
        form = SubjectConsentForm(
            data=data,
            initial=dict(screening_identifier=data.get("screening_identifier")),
            instance=SubjectConsent(),
        )
        form.is_valid()
        self.assertEqual({}, form._errors)

    def test_raises_on_duplicate_identity1(self):
        subject_consent = self.prepare_subject_consent(
            identity="1", confirm_identity="1", screening_identifier="LOPIKKKK"
        )
        opts = SubjectConsentForm._meta
        data = model_to_dict(subject_consent, opts.fields, opts.exclude)
        form = SubjectConsentForm(
            data=data,
            initial=dict(screening_identifier=data.get("screening_identifier")),
            instance=SubjectConsent(),
        )
        form.is_valid()
        self.assertEqual({}, form._errors)
        form.save(commit=True)

        subject_consent = self.prepare_subject_consent(
            identity="1", confirm_identity="1", screening_identifier="LOPIKXSWE"
        )
        opts = SubjectConsentForm._meta
        data = model_to_dict(subject_consent, opts.fields, opts.exclude)
        form = SubjectConsentForm(
            data=data,
            initial=dict(screening_identifier="LOPIKXSWE"),
            instance=subject_consent,
        )
        form.is_valid()
        self.assertIn("identity", form._errors)



from django.core.exceptions import ObjectDoesNotExist
from django.db import models

from .site_consents import SiteConsentError, site_consents


class ObjectConsentManager(models.Manager):
    def get_by_natural_key(self, subject_identifier_as_pk):
        return self.get(subject_identifier_as_pk=subject_identifier_as_pk)


class ConsentManager(models.Manager):
    def first_consent(self, subject_identifier=None):
        """Returns the first consent by consent_datetime."""
        return (
            self.filter(subject_identifier=subject_identifier)
            .order_by("consent_datetime")
            .first()
        )

    def consent_for_period(self, subject_identifier=None, report_datetime=None):
        """Returns a consent model instance or None."""
        model_obj = None
        try:
            consent_object = site_consents.get_consent_for_period(
                model=self.model._meta.label_lower,
                consent_group=self.model._meta.consent_group,
                report_datetime=report_datetime,
            )
        except SiteConsentError:
            pass
        else:
            try:
                model_obj = self.get(
                    subject_identifier=subject_identifier,
                    version=consent_object.version,
                )
            except ObjectDoesNotExist:
                pass
        return model_obj



from django.core.exceptions import ObjectDoesNotExist
from django.db.models.signals import pre_save
from django.dispatch import receiver
from edc_registration.models import RegisteredSubject
from edc_visit_schedule.site_visit_schedules import site_visit_schedules

from ..exceptions import NotConsentedError
from ..requires_consent import RequiresConsent


@receiver(pre_save, weak=False, dispatch_uid="requires_consent_on_pre_save")
def requires_consent_on_pre_save(instance, raw, using, update_fields, **kwargs):
    if not raw and not update_fields:
        try:
            consent_model = site_visit_schedules.all_post_consent_models[
                instance._meta.label_lower
            ]
        except KeyError:
            pass
        else:
            try:
                subject_identifier = instance.subject_identifier
            except AttributeError:
                subject_identifier = instance.related_visit.subject_identifier
            if consent_model:
                requires_consent = RequiresConsent(
                    model=instance._meta.label_lower,
                    subject_identifier=subject_identifier,
                    report_datetime=instance.report_datetime,
                    consent_model=consent_model,
                )
                instance.consent_version = requires_consent.version
            else:
                try:
                    RegisteredSubject.objects.get(
                        subject_identifier=subject_identifier,
                        consent_datetime__lte=instance.report_datetime,
                    )
                except ObjectDoesNotExist:
                    raise NotConsentedError(
                        f"Subject is not registered or was not registered by this date. "
                        f"Unable to save {instance._meta.label_lower}. "
                        f"Got {instance.subject_identifier} on "
                        f"{instance.report_datetime}."
                    )



from typing import Tuple

from django.core.handlers.wsgi import WSGIRequest

from ..utils import get_remove_patient_names_from_countries


class PiiNamesModelAdminMixin:
    """Will remove name fields from the modeladmin if the country is listed
    in EDC_CONSENT_REMOVE_PATIENT_NAMES_FROM_COUNTRIES.

    Should be first in the MRO"""

    name_fields: list[str] = ["first_name", "last_name"]
    name_display_field: str = "first_name"
    all_sites: dict = {}

    def get_fieldsets(self, request: WSGIRequest, obj=None) -> tuple:
        fieldsets = super().get_fieldsets(request, obj=obj)
        return self.fieldsets_without_names(fieldsets, request)

    def get_fields(self, request: WSGIRequest, obj=None) -> Tuple[str, ...]:
        fields = super().get_fields(request, obj=obj)
        return self.fields_without_names(fields, request)

    def get_search_fields(self, request: WSGIRequest) -> Tuple[str, ...]:
        search_fields = super().get_search_fields(request)
        return self.fields_without_names(search_fields, request)

    def get_list_display(self, request) -> tuple:
        fields = super().get_list_display(request)
        return self.fields_without_names(fields, request)

    def fieldsets_without_names(self, fieldsets: tuple, request: WSGIRequest) -> tuple:
        for country in get_remove_patient_names_from_countries():
            site = getattr(request, "site", None)
            if site and site.id in [s.site_id for s in self.all_sites.get(country)]:
                for fieldset in fieldsets:
                    fields = fieldset[1].get("fields")
                    fieldset[1]["fields"] = [f for f in fields if f not in self.name_fields]
        return fieldsets

    def fields_without_names(self, fields: tuple, request: WSGIRequest) -> Tuple[str, ...]:
        for country in get_remove_patient_names_from_countries():
            site = getattr(request, "site", None)
            if site and site.id in [s.site_id for s in self.all_sites.get(country)]:
                fields = tuple([f for f in fields if f not in self.name_fields])
        return fields



from django.views.generic.base import TemplateView
from edc_dashboard.utils import get_bootstrap_version
from edc_dashboard.view_mixins import EdcViewMixin
from edc_navbar import NavbarViewMixin

from ..admin_site import edc_consent_admin
from ..site_consents import site_consents


class HomeView(EdcViewMixin, NavbarViewMixin, TemplateView):

    template_name = f"edc_consent/bootstrap{get_bootstrap_version()}/home.html"
    navbar_name = "edc_consent"
    navbar_selected_item = "consent"

    def __init__(self, *args, **kwargs):
        super(HomeView, self).__init__(*args, **kwargs)

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context.update(edc_consent_admin=edc_consent_admin, consents=site_consents.consents)
        return context



from django.db import models
from edc_constants.choices import YES_NO, YES_NO_DECLINED

from ..validators import eligible_if_yes


class ScoredReviewFieldsMixin(models.Model):

    consent_reviewed = models.CharField(
        verbose_name="I have reviewed the consent with the client",
        max_length=3,
        choices=YES_NO,
        validators=[eligible_if_yes],
        null=True,
        blank=False,
        help_text="If no, INELIGIBLE",
    )

    study_questions = models.CharField(
        verbose_name="I have answered all questions the client had about the study",
        max_length=3,
        choices=YES_NO,
        validators=[eligible_if_yes],
        null=True,
        blank=False,
        help_text="If no, INELIGIBLE",
    )

    assessment_score = models.CharField(
        # TODO: i have asked the client questions about this study
        # and they have demonstrated understanding
        verbose_name=(
            "The client has completed the assessment of understanding with a passing score"
        ),
        max_length=3,
        choices=YES_NO,
        validators=[eligible_if_yes],
        null=True,
        blank=False,
        help_text="If no, INELIGIBLE",
    )

    consent_copy = models.CharField(
        verbose_name=(
            "I have provided the client with a copy of their signed informed" " edc_consent"
        ),
        max_length=3,
        choices=YES_NO_DECLINED,
        validators=[eligible_if_yes],
        null=True,
        blank=False,
        help_text=(
            "If no, INELIGIBLE. If declined, return copy to the " "clinic with the edc_consent"
        ),
    )

    class Meta:
        abstract = True



from django.conf import settings
from edc_protocol import Protocol
from edc_utils import convert_php_dateformat, floor_secs
from edc_utils.date import ceil_datetime, floor_datetime

from .exceptions import ConsentVersionSequenceError


class ConsentPeriodError(Exception):
    pass


class ConsentPeriodOverlapError(Exception):
    pass


class ConsentObjectValidator:
    def __init__(self, consent=None, consents=None):
        self.consents = consents
        self.check_consent_period_within_study_period(consent)
        self.check_consent_period_for_overlap(consent)
        self.check_version(consent)
        self.check_updates_versions(consent)

    def get_consents_by_model(self, model=None):
        """Returns a list of consents configured with the given
        consent model label_lower.
        """
        return [consent for consent in self.consents if consent.model == model]

    def get_consents_by_version(self, model=None, version=None):
        """Returns a list of consents of "version" configured with
        the given consent model.
        """
        consents = self.get_consents_by_model(model=model)
        return [consent for consent in consents if consent.version == version]

    def check_consent_period_for_overlap(self, new_consent=None):
        """Raises an error if consent period overlaps with an
        already registered consent object.
        """
        for consent in self.consents:
            if consent.model == new_consent.model:
                if (
                    new_consent.start <= consent.start <= new_consent.end
                    or new_consent.start <= consent.end <= new_consent.end
                ):
                    raise ConsentPeriodOverlapError(
                        f"Consent period overlaps with an already registered consent."
                        f"See alrwady registered consent {consent}. "
                        f"Got {new_consent}."
                    )

    @staticmethod
    def check_consent_period_within_study_period(new_consent=None):
        """Raises if the start or end date of the consent period
        it not within the opening and closing dates of the protocol.
        """
        protocol = Protocol()
        study_open_datetime = protocol.study_open_datetime
        study_close_datetime = protocol.study_close_datetime
        for index, dt in enumerate([new_consent.start, new_consent.end]):
            if not (
                floor_secs(floor_datetime(study_open_datetime))
                <= floor_secs(dt)
                <= floor_secs(ceil_datetime(study_close_datetime))
            ):
                dt_label = "start" if index == 0 else "end"
                formatted_study_open_datetime = study_open_datetime.strftime(
                    convert_php_dateformat(settings.SHORT_DATETIME_FORMAT)
                )
                formatted_study_close_datetime = study_close_datetime.strftime(
                    convert_php_dateformat(settings.SHORT_DATETIME_FORMAT)
                )
                formatted_dt = dt.strftime(
                    convert_php_dateformat(settings.SHORT_DATETIME_FORMAT)
                )
                raise ConsentPeriodError(
                    f"Invalid consent. Consent period for {new_consent.name} "
                    "must be within study opening/closing dates of "
                    f"{formatted_study_open_datetime} - "
                    f"{formatted_study_close_datetime}. "
                    f"Got {dt_label}={formatted_dt}."
                )

    def check_updates_versions(self, new_consent=None):
        for version in new_consent.updates_versions:
            if not self.get_consents_by_version(model=new_consent.model, version=version):
                raise ConsentVersionSequenceError(
                    f"Consent version {version} cannot be an update to version(s) "
                    f"'{new_consent.updates_versions}'. "
                    f"Version '{version}' not found for '{new_consent.model}'"
                )

    def check_version(self, new_consent=None):
        if self.get_consents_by_version(model=new_consent.model, version=new_consent.version):
            raise ConsentVersionSequenceError(
                "Consent version already registered. "
                f"Version {new_consent.version}. "
                f"Got {new_consent}."
            )
